/*
Copyright Renaud GICQUEL 2005-2009 
renaud.gicquel@ensmp.fr

Ce logiciel est un programme informatique servant  modliser des systmes nergtiques.   

Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA  sur le site "http://www.cecill.info".  

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons, seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et les concdants successifs.  

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,   la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant  donn sa spcificit de logiciel libre, qui peut le rendre complexe   manipuler et qui le rserve donc  des dveloppeurs et des professionnels avertis possdant  des  connaissances  informatiques approfondies.  Les utilisateurs sont donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,   l'utiliser et l'exploiter dans les mmes conditions de scurit.   

Le fait que vous puissiez accder  cet en-tte signifie que vous avez  pris connaissance de la licence CeCILL, et que vous en avez accept les termes. 

This software is a computer program whose purpose is to model energy systems.  

This software is governed by the CeCILL  license under French law and abiding by the rules of distribution of free software.  You can  use,  modify and/ or redistribute the software under the terms of the CeCILL license as circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".   

As a counterpart to the access to the source code and  rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty  and the software's author,  the holder of the economic rights,  and the successive licensors  have only  limited liability.   

In this respect, the user's attention is drawn to the risks associated with loading,  using,  modifying and/or developing or reproducing the software by the user in light of its specific status of free software, that may mean  that it is complicated to manipulate,  and  that  also therefore means  that it is reserved for developers  and  experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the software's suitability as regards their requirements in conditions enabling the security of their systems and/or  data to be ensured and,  more generally, to use and operate it in the  same conditions as regards security.   

The fact that you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.  
*/
package extThopt;

import extThopt.*;
import java.util.*; 
/**
* This class is an example of an external mixture for Thermoptim. 
*
* @author Renaud GICQUEL
*/

public class LiBrH2OMixture extends ExtSubstance 
{
    
    
    public LiBrH2OMixture(){//throws Exception{
        super();
        type=getType();
        M=29;PC=10;TC=350; 
        Tmini=278.15;  Tmaxi=448.15; 
        chemForm="LiBr-H2O mixture";
        typeCorps=6;//corps externe de type isMixture

  	    vMixtures= new Vector();
    }
    
    public String getType(){
        return "LiBr-H2O mixture";
    }
    
	public double getSatPressure(double T, double x){
	    return getP(x, T);
	}
    
	public double calcXx (double T, double P,double x) {
	    return x;
	}
    
    /** Calculates the volume of LiBr-H2O solution
    * reference : ASHRAE Toolkit 
    *@param x mass fraction of LiBr (between 0 and 1) and then converted in X between 40 and 100
    *@param T solution temperature (K)
    *@return P (bar)
    */
	public double calcV (double T, double P,double x) {
	    double X=100.*(1-x);
	    return 1.0111e-3-7.1622e-6*X;
	}
	
    /** Calculates the crystallization limit enthalpy of LiBr-H2O solution
    * reference : ASHRAE Toolkit 
    *@param x mass fraction of LiBr (between 0 and 1) and then converted in X between 40 and 100
    *@param T solution temperature (K)
    *@return P (bar)
    */
	private double getCrystallizationLimitH(double x) {
	    double X=100.*(100-x);
	    return -139667+24000*X;
	}
    
    /** Calculates the pressure of LiBr-H2O solution
    * reference : ASHRAE, Fundamentals Handbook (SI), 
    * Thermophysical properties of refrigerants, p. 20.67, 2001
    *@param x mass fraction of refrigerant (H2O) (between 0 and 1)
    *@param T solution temperature (K)
    *@return P (bar)
    */
	private double getP(double x,double T){
	    double X=100.*(1-x);//ASHRAE properties are given for X
	    double t=T-273.15;
	    double[] A=new double[4];
	    double[] B=new double[4];
        A[0]=-2.00755;A[1]=0.16976;A[2]=-0.003133362;A[3]=0.0000197668;
	    B[0]=124.93700000;B[1]=-7.71649;B[2]=0.15228600;B[3]=-0.00079509;
        double C=7.05,D=-1596.49,E=-104095.5;
        double Tprime,sigAi=0,sigBi=0;
        for(int i=0;i<4;i++){
            sigAi=sigAi+A[i]*Math.pow(X, i);
            sigBi=sigBi+B[i]*Math.pow(X, i);
        }
        Tprime=(t-sigBi)/sigAi+273.15;
        
        double logP=C+D/Tprime+E/Tprime/Tprime;
        return Math.exp(logP/0.43429)/100;
        
    /*     double Ax=29.97-0.091*X;
        return Math.exp(Ax-5371/T)/100000.;*/

	}
	
	public double calcH (double T, double P,double x) {
	    return getH(x, T);
	}

    /** Calculates the enthalpy of LiBr-H2O solution
    * reference : ASHRAE, Fundamentals Handbook (SI), 
    * Thermophysical properties of refrigerants, p. 20.66, 2001
    *@param x mass fraction of refrigerant (H2O) (between 0 and 1)
    *@param T solution temperature (K)
    *@return h (kJ/kg/K)
    */
    	private double getH(double x,double T){
	    double X=100.*(1-x);//ASHRAE properties are given for X
	    double[] A=new double[5];
	    double[] B=new double[5];
	    double[] C=new double[5];
        A[0]=-2024.33;A[1]=163.309;A[2]=-4.88161;A[3]=0.06302948;A[4]=-0.0002913705;
        B[0]=18.2829;B[1]=-1.16917570;B[2]=0.03248041;B[3]=-0.0004034184;B[4]=0.00000185205690;
        C[0]=-0.03700821;C[1]=0.0028877666;C[2]=-0.000081313015;C[3]=0.000000991166280;C[4]=-0.000000004444120700;
        double sigAi=0,sigBi=0,sigCi=0;
        for(int i=0;i<5;i++){
            sigAi=sigAi+A[i]*Math.pow(X, i);
            sigBi=sigBi+B[i]*Math.pow(X, i);
            sigCi=sigCi+C[i]*Math.pow(X, i);
        }
        
        
        molarWeights=new double[6];
      	molarWeights[0]=M;
      	molarWeights[1]=1;
      	molarWeights[2]=1;
      	molarWeights[3]=Tmini;
      	molarWeights[4]=Tmaxi;
      	molarWeights[5]=1000;
        
        return sigAi+(T-273.15)*sigBi+(T-273.15)*(T-273.15)*sigCi;

	}


}